define(function (require) {
  var App = require('app');

  var months = ['January', 'February', 'March', 'April', 'May', 'June', 'July', 'August', 'September', 'October', 'November', 'December'];
  var monthsShort = ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'];
  var days = ['Sunday', 'Monday', 'Tuesday', 'Wednesday', 'Thursday', 'Friday', 'Saturday', 'Sunday'];
  var daysShort = ['Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat'];
  var daysMin = ['Su', 'Mo', 'Tu', 'We', 'Th', 'Fr', 'Sa', 'Su'];
  var formatParts = /dd?|DD?|mm?|MM?|yy(?:yy)?/g;

  var _MS_PER_WEEK = 1000 * 60 * 60 * 24 * 7;

  var _DEFAULT_DATE = new Date(-2209057200000);
  var _GESTATION = 280;

  return Backbone.Marionette.Controller.extend({
    /* Static variable retrieval */
    getGestation: function () { return _GESTATION; },
    getDefaultDate: function () { return _DEFAULT_DATE; },

    /* Date Generation */

    getZeroHourDate: function (date) {
      if (!date) {
        date = new Date();
      }
      date.setHours(0, 0, 0, 0);
      return date;
    },

    /* Date Conversions */

    toUTC: function (date) {
      return Date.UTC(date.getFullYear(), date.getMonth(), date.getDate(), date.getHours(), date.getMinutes(),
        date.getSeconds(), date.getMilliseconds());
    },
    convertDate: function (date, format) {
      if (!(date instanceof Date)) { return ''; }
      if (!format) { format = 'mm/dd/yyyy'; }
      if (typeof format === 'string') {
        format = this.parseFormat(format);
      }
      var val = {
        d: date.getUTCDate(),
        D: daysShort[date.getUTCDay()],
        DD: days[date.getUTCDay()],
        m: date.getUTCMonth() + 1,
        M: monthsShort[date.getUTCMonth()],
        MM: months[date.getUTCMonth()],
        yy: date.getUTCFullYear().toString().substring(2),
        yyyy: date.getUTCFullYear()
      };
      val.dd = (val.d < 10 ? '0' : '') + val.d;
      val.mm = (val.m < 10 ? '0' : '') + val.m;
      date = [];
      var seps = $.extend([], format.separators);
      for (var i = 0, cnt = format.parts.length; i <= cnt; i++) {
        if (seps.length) {
          date.push(seps.shift());
        }
        date.push(val[format.parts[i]]);
      }
      return date.join('');
    },
    parseFormat: function (format) {
      var separators = format.replace(formatParts, '\0').split('\0');
      var parts = format.match(formatParts);
      if (!separators || !separators.length || !parts || parts.length === 0) {
        throw new Error('Invalid date format.');
      }
      return {separators: separators, parts: parts};
    },
    /* Date Validations */

    validateDate: function (datestr) {
      if (datestr instanceof Date) {
        return datestr;
      }
      if (typeof datestr === 'number') {
        return new Date(datestr);
      }
      var day, datearr;
      datearr = datestr.split(/\W+/).map(function (itm) {
        if ($.isNumeric(itm)) {
          return parseInt(itm, 10);
        }
        else {
          for (var i = 0; i < months.length; i++) {
            if (itm.toLowerCase().indexOf(monthsShort[i].toLowerCase()) > -1 ||
              itm.toLowerCase().indexOf(months[i].toLowerCase()) > -1) {
              return i + 1;
            }
          }
          return NaN;
        }
      });
      try {
        day = new Date(datearr[2], datearr[0] - 1, datearr[1]);
        if (day.getUTCMonth() + 1 === datearr[0] && day.getUTCDate() === datearr[1]) {
          return day;
        }
        throw 'Bad Date Format';
      }
      catch (err) {
        return NaN;
      }
    },
    isDefaultDate: function (date) {
      if (date === _DEFAULT_DATE.getTime() || date === _DEFAULT_DATE.toISOString()) {
        return true;
      }
      else if (date instanceof Date) {
        return (+date === +_DEFAULT_DATE);
      }
      else { return false; }
    },

    /* DateForm events (Bootstrap) */

    changeDate: function (e) {
      var calendar = this.convertDate(e.date);
      $(e.currentTarget).parent().children('.dateform').val(calendar).trigger('change');
    },
    dateFieldIsValid: function (field) {
      var val = field.val();
      var yearCheck = val.match(/\/\d\d$/g);
      if (yearCheck !== null) {
        yearCheck = yearCheck[0].substring(1);
        val = val.replace(/\/\d\d$/g, '/20' + yearCheck);
        field.val(val);
      }
      var dat = this.validateDate(val);
      var $helpSpace = field.parent().siblings('.help-block').children('.date-validate').first();
      if (dat) {
        var helpText = App.router.dates.convertDate(dat, 'D mm/dd/yyyy');
        $helpSpace.removeClass('text-danger').attr('aria-live', 'polite')
          .text(helpText);
        return true;
      } else if (field.val().length > 0) {
        $helpSpace.addClass('text-danger').attr('aria-live', 'assertive')
          .text('Not a valid date.');
        return false;
      } else {
        $helpSpace.removeClass('text-danger').attr('aria-live', 'polite')
          .html('&nbsp');
        return false;
      }
    },

    /* Calculations with Dates */

    addDays: function (date, numdays) {
      if (date) {
        return date.addDays(numdays);
      }
    },
    dateDiffInWeeks: function (a, b) {
      var utc1 = Date.UTC(a.getUTCFullYear(), a.getUTCMonth(), a.getUTCDate());
      var utc2 = Date.UTC(b.getUTCFullYear(), b.getUTCMonth(), b.getUTCDate());

      return Math.ceil((utc2 - utc1) / _MS_PER_WEEK);
    },
    calculateAge: function (date) {
      var today = this.getZeroHourDate();
      var age = today.getUTCFullYear() - date.getUTCFullYear();
      var m = today.getUTCMonth() - date.getUTCMonth();
      if ((m < 0) || (m === 0 && today.getUTCDate() < date.getUTCDate())) {
        age--;
      }
      return age;
    },
    getWeekAndDays: function (a, b) {
      var msInDay = 1000 * 60 * 60 * 24;

      var utc1 = Date.UTC(a.getUTCFullYear(), a.getUTCMonth(), a.getUTCDate());
      var utc2 = Date.UTC(b.getUTCFullYear(), b.getUTCMonth(), b.getUTCDate());

      var diff = Math.floor((utc2 - utc1) / msInDay);
      var str = '';
      var weeks = 0;
      var values = {
        ' Week': 7,
        ' Day': 1
      };
      for (var x in values) {
        var amount = Math.floor(diff / values[x]);
        if (amount >= 1) {
          if (x === ' Week') {
            str += amount + x + (amount > 1 ? 's' : '') + ' ';
            weeks = amount;
          } else {
            str += (weeks === 0 ? '' : 'and ') + amount + x + (amount > 1 ? 's' : '') + ' ';
          }
          diff -= amount * values[x];
        }
      }
      return {string: str, weeks: weeks};
    }
  });
});

Date.prototype.addDays = function (numdays) {
  var dat = new Date(this.valueOf());
  dat.setDate(dat.getUTCDate() + numdays);
  return dat;
};
